@@ -17,8 +17,15 @@ search_files <- function(file, dir = ".", recursive = TRUE)
 
 include_supplement <- function(file, dir = NULL, recursive = FALSE) {
   if(is.null(dir)) {
-    dir <- try(.xexams_dir$exercises, silent = TRUE)
-    if(inherits(dir, "try-error") | is.null(dir)) {
+    dir <- try(.exams_get_internal("xexams_dir_exercises"), silent = TRUE)
+    if(inherits(dir, "try-error") | is.null(dir)) dir <- getwd()
+  } else {
+    if(!file.exists(dir)) {
+      odir <- dir
+      dir <- try(file.path(.exams_get_internal("xexams_dir_exercises"), dir), silent = TRUE)
+    }
+    if(inherits(dir, "try-error") | !file.exists(dir)) {
+      warning(sprintf("The 'dir' could not be found: %s", odir))
       dir <- getwd()
     }
   }
@@ -34,8 +41,11 @@ include_supplement <- function(file, dir = NULL, recursive = FALSE) {
 
 match_exams_call <- function(which = 1L, deparse = TRUE) {
   if(getRversion() < "3.2.0") return("")
-  rval <- if(!deparse) .xexams_call else sapply(.xexams_call, function(x) deparse(x[[1L]]))
+  rval <- .exams_get_internal("xexams_call")
+  if(deparse) rval <- sapply(rval, function(x) deparse(x[[1L]], width.cutoff = 500))
   if(!is.null(which)) rval <- rval[[which]]
-  if(rval == "NULL") rval <- ""
+  rval[rval == "NULL"] <- ""
   return(rval)
 }
+
+match_exams_device <- function() .exams_get_internal("xweave_device")
